@@ -1,20 +1,20 @@
 /*
- * Licensed to the Apache Software Foundation (ASF) under one
- * or more contributor license agreements.  See the NOTICE file
- * distributed with this work for additional information
- * regarding copyright ownership.  The ASF licenses this file
- * to you under the Apache License, Version 2.0 (the
- * "License"); you may not use this file except in compliance
- * with the License.  You may obtain a copy of the License at
- *
- *   http://www.apache.org/licenses/LICENSE-2.0
- *
- * Unless required by applicable law or agreed to in writing,
- * software distributed under the License is distributed on an
- * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
- * KIND, either express or implied.  See the License for the
- * specific language governing permissions and limitations
- * under the License.
+ *  Licensed to the Apache Software Foundation (ASF) under one
+ *  or more contributor license agreements.  See the NOTICE file
+ *  distributed with this work for additional information
+ *  regarding copyright ownership.  The ASF licenses this file
+ *  to you under the Apache License, Version 2.0 (the
+ *  "License"); you may not use this file except in compliance
+ *  with the License.  You may obtain a copy of the License at
+ * 
+ *  http://www.apache.org/licenses/LICENSE-2.0
+ * 
+ *  Unless required by applicable law or agreed to in writing,
+ *  software distributed under the License is distributed on an
+ *  "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+ *  KIND, either express or implied.  See the License for the
+ *  specific language governing permissions and limitations
+ *  under the License.
  */
 package org.apache.myfaces.shared_tomahawk.renderkit.html;
 
@@ -22,9 +22,9 @@ import org.apache.myfaces.shared_tomahawk.renderkit.JSFAttr;
 import org.apache.myfaces.shared_tomahawk.renderkit.RendererUtils;
 import org.apache.myfaces.shared_tomahawk.renderkit.html.util.FormInfo;
 import org.apache.myfaces.shared_tomahawk.renderkit.html.util.JavascriptUtils;
+import org.apache.myfaces.shared_tomahawk.util._ComponentUtils;
 import org.apache.myfaces.shared_tomahawk.config.MyfacesConfig;
 
-import javax.faces.application.StateManager;
 import javax.faces.application.ViewHandler;
 import javax.faces.component.UICommand;
 import javax.faces.component.UIComponent;
@@ -41,58 +41,71 @@ import java.util.Iterator;
 
 /**
  * @author Manfred Geiler
- * @version $Revision: 947902 $ $Date: 2010-05-24 22:39:45 -0500 (Mon, 24 May 2010) $
+ * @version $Revision: 947903 $ $Date: 2010-05-24 22:40:23 -0500 (Mon, 24 May 2010) $
  */
 public abstract class HtmlLinkRendererBase
-    extends HtmlRenderer {
-    public static final String URL_STATE_MARKER = "JSF_URL_STATE_MARKER=DUMMY";
-    public static final int URL_STATE_MARKER_LEN = URL_STATE_MARKER.length();
+    extends HtmlRenderer
+{
+    /* this one is never used
+    public static final String URL_STATE_MARKER      = "JSF_URL_STATE_MARKER=DUMMY";
+    public static final int    URL_STATE_MARKER_LEN  = URL_STATE_MARKER.length();
+    */
 
     //private static final Log log = LogFactory.getLog(HtmlLinkRenderer.class);
 
-    public boolean getRendersChildren() {
+    public boolean getRendersChildren()
+    {
         // We must be able to render the children without a surrounding anchor
         // if the Link is disabled
         return true;
     }
 
-    public void decode(FacesContext facesContext, UIComponent component) {
+    public void decode(FacesContext facesContext, UIComponent component)
+    {
         super.decode(facesContext, component);  //check for NP
 
-        if (component instanceof UICommand) {
+        if (component instanceof UICommand)
+        {
             String clientId = component.getClientId(facesContext);
             FormInfo formInfo = findNestingForm(component, facesContext);
             String reqValue = (String) facesContext.getExternalContext().getRequestParameterMap().get(
                 HtmlRendererUtils.getHiddenCommandLinkFieldName(formInfo));
-            if (reqValue != null && reqValue.equals(clientId)) {
+            if (reqValue != null && reqValue.equals(clientId))
+            {
                 component.queueEvent(new ActionEvent(component));
 
                 RendererUtils.initPartialValidationAndModelUpdate(component, facesContext);
             }
         }
-        else if (component instanceof UIOutput) {
+        else if (component instanceof UIOutput)
+        {
             //do nothing
         }
-        else {
+        else
+        {
             throw new IllegalArgumentException("Unsupported component class " + component.getClass().getName());
         }
     }
 
 
-    public void encodeBegin(FacesContext facesContext, UIComponent component) throws IOException {
+    public void encodeBegin(FacesContext facesContext, UIComponent component) throws IOException
+    {
         super.encodeBegin(facesContext, component);  //check for NP
 
-        if (component instanceof UICommand) {
+        if (component instanceof UICommand)
+        {
             renderCommandLinkStart(facesContext, component,
                                    component.getClientId(facesContext),
-                                   ((UICommand) component).getValue(),
+                                   ((UICommand)component).getValue(),
                                    getStyle(facesContext, component),
                                    getStyleClass(facesContext, component));
         }
-        else if (component instanceof UIOutput) {
-            renderOutputLinkStart(facesContext, (UIOutput) component);
+        else if (component instanceof UIOutput)
+        {
+            renderOutputLinkStart(facesContext, (UIOutput)component);
         }
-        else {
+        else
+        {
             throw new IllegalArgumentException("Unsupported component class " + component.getClass().getName());
         }
     }
@@ -101,44 +114,53 @@ public abstract class HtmlLinkRendererBase
     /**
      * Can be overwritten by derived classes to overrule the style to be used.
      */
-    protected String getStyle(FacesContext facesContext, UIComponent link) {
-        if (link instanceof HtmlCommandLink) {
-            return ((HtmlCommandLink) link).getStyle();
-        }
-        else {
-            return (String) link.getAttributes().get(HTML.STYLE_ATTR);
+    protected String getStyle(FacesContext facesContext, UIComponent link)
+    {
+        if (link instanceof HtmlCommandLink)
+        {
+            return ((HtmlCommandLink)link).getStyle();
         }
+
+        return (String)link.getAttributes().get(HTML.STYLE_ATTR);
+
     }
 
     /**
      * Can be overwritten by derived classes to overrule the style class to be used.
      */
-    protected String getStyleClass(FacesContext facesContext, UIComponent link) {
-        if (link instanceof HtmlCommandLink) {
-            return ((HtmlCommandLink) link).getStyleClass();
-        }
-        else {
-            return (String) link.getAttributes().get(HTML.STYLE_CLASS_ATTR);
+    protected String getStyleClass(FacesContext facesContext, UIComponent link)
+    {
+        if (link instanceof HtmlCommandLink)
+        {
+            return ((HtmlCommandLink)link).getStyleClass();
         }
+
+        return (String)link.getAttributes().get(HTML.STYLE_CLASS_ATTR);
+
     }
 
-    public void encodeChildren(FacesContext facesContext, UIComponent component) throws IOException {
+    public void encodeChildren(FacesContext facesContext, UIComponent component) throws IOException
+    {
         RendererUtils.renderChildren(facesContext, component);
     }
 
-    public void encodeEnd(FacesContext facesContext, UIComponent component) throws IOException {
+    public void encodeEnd(FacesContext facesContext, UIComponent component) throws IOException
+    {
         super.encodeEnd(facesContext, component);  //check for NP
 
-        if (component instanceof UICommand) {
-            renderCommandLinkEnd(facesContext, component);
+        if (component instanceof UICommand)
+        {
+                renderCommandLinkEnd(facesContext, component);
 
             HtmlFormRendererBase.renderScrollHiddenInputIfNecessary(
                 findNestingForm(component, facesContext).getForm(), facesContext, facesContext.getResponseWriter());
         }
-        else if (component instanceof UIOutput) {
+        else if (component instanceof UIOutput)
+        {
             renderOutputLinkEnd(facesContext, component);
         }
-        else {
+        else
+        {
             throw new IllegalArgumentException("Unsupported component class " + component.getClass().getName());
         }
     }
@@ -148,29 +170,42 @@ public abstract class HtmlLinkRendererBase
                                           Object value,
                                           String style,
                                           String styleClass)
-        throws IOException {
+            throws IOException
+    {
         ResponseWriter writer = facesContext.getResponseWriter();
 
-        String[] anchorAttrsToRender;
-        if (JavascriptUtils.isJavascriptAllowed(facesContext.getExternalContext())) {
-            renderJavaScriptAnchorStart(facesContext, writer, component, clientId);
-            anchorAttrsToRender = HTML.ANCHOR_PASSTHROUGH_ATTRIBUTES_WITHOUT_ONCLICK_WITHOUT_STYLE;
-        }
-        else {
-            renderNonJavaScriptAnchorStart(facesContext, writer, component, clientId);
-            anchorAttrsToRender = HTML.ANCHOR_PASSTHROUGH_ATTRIBUTES_WITHOUT_STYLE;
+        if (HtmlRendererUtils.isDisabled(component))
+        {
+            writer.startElement(HTML.SPAN_ELEM, component);
+            HtmlRendererUtils.writeIdIfNecessary(writer, component, facesContext);
+            HtmlRendererUtils.renderHTMLAttributes(writer, component, HTML.ANCHOR_PASSTHROUGH_ATTRIBUTES);
         }
+        else
+        {
+            String[] anchorAttrsToRender;
+            if (JavascriptUtils.isJavascriptAllowed(facesContext.getExternalContext()))
+            {
+                renderJavaScriptAnchorStart(facesContext, writer, component, clientId);
+                anchorAttrsToRender = HTML.ANCHOR_PASSTHROUGH_ATTRIBUTES_WITHOUT_ONCLICK_WITHOUT_STYLE;
+            }
+            else
+            {
+                renderNonJavaScriptAnchorStart(facesContext, writer, component, clientId);
+                anchorAttrsToRender = HTML.ANCHOR_PASSTHROUGH_ATTRIBUTES_WITHOUT_STYLE;
+            }
 
-        writer.writeAttribute(HTML.ID_ATTR, clientId, null);
-        HtmlRendererUtils.renderHTMLAttributes(writer, component,
-                                               anchorAttrsToRender);
-        HtmlRendererUtils.renderHTMLAttribute(writer, HTML.STYLE_ATTR, HTML.STYLE_ATTR,
-                                              style);
-        HtmlRendererUtils.renderHTMLAttribute(writer, HTML.STYLE_CLASS_ATTR, HTML.STYLE_CLASS_ATTR,
-                                              styleClass);
+            HtmlRendererUtils.writeIdIfNecessary(writer, component, facesContext);
+            HtmlRendererUtils.renderHTMLAttributes(writer, component,
+                                                   anchorAttrsToRender);
+            HtmlRendererUtils.renderHTMLAttribute(writer, HTML.STYLE_ATTR, HTML.STYLE_ATTR,
+                                                  style);
+            HtmlRendererUtils.renderHTMLAttribute(writer, HTML.STYLE_CLASS_ATTR, HTML.STYLE_CLASS_ATTR,
+                                                  styleClass);
+        }
 
         // render value as required by JSF 1.1 renderkitdocs
-        if (value != null) {
+        if(value != null)
+        {
             writer.writeText(value.toString(), JSFAttr.VALUE_ATTR);
         }
     }
@@ -179,10 +214,12 @@ public abstract class HtmlLinkRendererBase
                                                ResponseWriter writer,
                                                UIComponent component,
                                                String clientId)
-        throws IOException {
+        throws IOException
+    {
         //Find form
         FormInfo formInfo = findNestingForm(component, facesContext);
-        if (formInfo == null) {
+        if (formInfo == null)
+        {
             String path = RendererUtils.getPathToComponent(component);
             String msg = "Link is not embedded in a form. Change component/tag '" + clientId + "' from javax.faces.*/<h:tagName /> " +
                 "to org.apache.myfaces.*/<t:tagName />, or embed it in a form.  This is not a bug. " +
@@ -197,15 +234,21 @@ public abstract class HtmlLinkRendererBase
         StringBuffer onClick = new StringBuffer();
 
         String commandOnclick;
-        if (component instanceof HtmlCommandLink) {
-            commandOnclick = ((HtmlCommandLink) component).getOnclick();
+        if (component instanceof HtmlCommandLink)
+        {
+            commandOnclick = ((HtmlCommandLink)component).getOnclick();
         }
-        else {
-            commandOnclick = (String) component.getAttributes().get(HTML.ONCLICK_ATTR);
+        else
+        {
+            commandOnclick = (String)component.getAttributes().get(HTML.ONCLICK_ATTR);
         }
-        if (commandOnclick != null) {
+        if (commandOnclick != null)
+        {
+            onClick.append("var cf = function(){");
             onClick.append(commandOnclick);
+            onClick.append('}');
             onClick.append(';');
+            onClick.append("var oamSF = function(){");
         }
 
         if (RendererUtils.isAdfOrTrinidadForm(formInfo.getForm())) {
@@ -213,15 +256,12 @@ public abstract class HtmlLinkRendererBase
             onClick.append(formInfo.getForm().getClientId(facesContext));
             onClick.append("',1,{source:'");
             onClick.append(component.getClientId(facesContext));
-            onClick.append("'");
-            // See Jira Issue 1809 https://issues.apache.org/jira/browse/MYFACES-1809
-            onClick.append(getChildParametersTrinidad(component));
-            onClick.append("});return false;");
+            onClick.append("'});return false;");
         }
         else {
             HtmlRendererUtils.renderFormSubmitScript(facesContext);
 
-            StringBuffer params = addChildParameters(component, nestingForm);
+            StringBuffer params = addChildParameters(facesContext, component, nestingForm);
 
             String target = getTarget(component);
 
@@ -237,15 +277,22 @@ public abstract class HtmlLinkRendererBase
             }
             onClick.append(");");
 
-
+            //Not necessary since we are using oamSetHiddenInput to create input hidden fields
             //render hidden field - todo: in here for backwards compatibility
-            String hiddenFieldName = HtmlRendererUtils.getHiddenCommandLinkFieldName(formInfo);
-            addHiddenCommandParameter(facesContext, nestingForm, hiddenFieldName);
+            if (MyfacesConfig.getCurrentInstance(facesContext.getExternalContext()).isRenderHiddenFieldsForLinkParams())
+            {
+                String hiddenFieldName = HtmlRendererUtils.getHiddenCommandLinkFieldName(formInfo);
+                addHiddenCommandParameter(facesContext, nestingForm, hiddenFieldName);
+            }
 
-            //render hidden field - todo: in here for backwards compatibility
-            String hiddenFieldNameMyFacesOld = HtmlRendererUtils.getHiddenCommandLinkFieldNameMyfacesOld(formInfo);
-            addHiddenCommandParameter(facesContext, nestingForm, hiddenFieldNameMyFacesOld);
         }
+        
+        if (commandOnclick != null)
+        {
+            onClick.append('}');
+            onClick.append(';');
+            onClick.append("return (cf()==false)? false : oamSF();");
+        }        
 
         writer.startElement(HTML.ANCHOR_ELEM, component);
         writer.writeURIAttribute(HTML.HREF_ATTR, "#", null);
@@ -264,39 +311,7 @@ public abstract class HtmlLinkRendererBase
         return target;
     }
 
-   /**
-    * See Jira Issue 1809 https://issues.apache.org/jira/browse/MYFACES-1809
-    */
-    private String getChildParametersTrinidad(UIComponent component) {
-        //https://issues.apache.org/jira/browse/MYFACES-1809//add child parameters
-        StringBuffer params = new StringBuffer();
-        for (Iterator it = getChildren(component).iterator(); it.hasNext();) {
-
-            UIComponent child = (UIComponent) it.next();
-            if (child instanceof UIParameter) {
-                String name = ((UIParameter) child).getName();
-
-                if (name == null) {
-                   throw new IllegalArgumentException("Unnamed parameter value not allowed within command link.");
-                }
-
-                Object value = ((UIParameter) child).getValue();
-
-                //UIParameter is no ValueHolder, so no conversion possible - calling .toString on value....
-                String strParamValue = value != null ? org.apache.myfaces.shared_tomahawk.renderkit.html.util.HTMLEncoder.encode(value.toString(), false, false) : "";
-
-                params.append(",'");
-
-                params.append(name);
-                params.append("':'");
-                params.append(strParamValue);
-                params.append("'");
-            }
-        }
-        return params.toString();
-    }
-
-    private StringBuffer addChildParameters(UIComponent component, UIComponent nestingForm) {
+    private StringBuffer addChildParameters(FacesContext context, UIComponent component, UIComponent nestingForm) {
         //add child parameters
         StringBuffer params = new StringBuffer();
         params.append("[");
@@ -310,7 +325,11 @@ public abstract class HtmlLinkRendererBase
                     throw new IllegalArgumentException("Unnamed parameter value not allowed within command link.");
                 }
 
-                addHiddenCommandParameter(FacesContext.getCurrentInstance(), nestingForm, name);
+                //Not necessary, since we are using oamSetHiddenInput to create hidden fields
+                if (MyfacesConfig.getCurrentInstance(context.getExternalContext()).isRenderHiddenFieldsForLinkParams())
+                {
+                    addHiddenCommandParameter(context, nestingForm, name);
+                }
 
                 Object value = ((UIParameter) child).getValue();
 
@@ -373,12 +392,15 @@ public abstract class HtmlLinkRendererBase
      * find nesting form<br />
      * need to be overrideable to deal with dummyForm stuff in tomahawk.
      */
-    protected FormInfo findNestingForm(UIComponent uiComponent, FacesContext facesContext) {
-        return RendererUtils.findNestingForm(uiComponent, facesContext);
+    protected FormInfo findNestingForm(UIComponent uiComponent, FacesContext facesContext)
+    {
+        return _ComponentUtils.findNestingForm(uiComponent, facesContext);
     }
 
-    protected void addHiddenCommandParameter(FacesContext facesContext, UIComponent nestingForm, String hiddenFieldName) {
-        if (nestingForm != null) {
+    protected void addHiddenCommandParameter(FacesContext facesContext, UIComponent nestingForm, String hiddenFieldName)
+    {
+        if (nestingForm != null)
+        {
             HtmlFormRendererBase.addHiddenCommandParameter(facesContext, nestingForm, hiddenFieldName);
         }
     }
@@ -388,7 +410,8 @@ public abstract class HtmlLinkRendererBase
                                                   ResponseWriter writer,
                                                   UIComponent component,
                                                   String clientId)
-        throws IOException {
+        throws IOException
+    {
         ViewHandler viewHandler = facesContext.getApplication().getViewHandler();
         String viewId = facesContext.getViewRoot().getViewId();
         String path = viewHandler.getActionURL(facesContext, viewId);
@@ -400,10 +423,12 @@ public abstract class HtmlLinkRendererBase
 
         //add clientId parameter for decode
 
-        if (path.indexOf('?') == -1) {
+        if (path.indexOf('?') == -1)
+        {
             hrefBuf.append('?');
         }
-        else {
+        else
+        {
             if (strictXhtmlLinks) {
                 hrefBuf.append("&amp;");
             } else {
@@ -415,21 +440,13 @@ public abstract class HtmlLinkRendererBase
         hrefBuf.append('=');
         hrefBuf.append(clientId);
 
-        if (getChildCount(component) > 0) {
+        if (getChildCount(component) > 0)
+        {
             addChildParametersToHref(facesContext, component, hrefBuf,
                                      false, //not the first url parameter
                                      writer.getCharacterEncoding());
         }
 
-        StateManager stateManager = facesContext.getApplication().getStateManager();
-        if (strictXhtmlLinks) {
-            hrefBuf.append("&amp;");
-        } else {
-            hrefBuf.append('&');
-        }
-        if (stateManager.isSavingStateInClient(facesContext)) {
-            hrefBuf.append(URL_STATE_MARKER);
-        }
         String href = facesContext.getExternalContext().encodeActionURL(hrefBuf.toString());
         writer.startElement(HTML.ANCHOR_ELEM, component);
         writer.writeURIAttribute(HTML.HREF_ATTR,
@@ -441,14 +458,18 @@ public abstract class HtmlLinkRendererBase
                                           UIComponent linkComponent,
                                           StringBuffer hrefBuf,
                                           boolean firstParameter,
-                                          String charEncoding) throws IOException {
+                                          String charEncoding)
+            throws IOException
+    {
         boolean strictXhtmlLinks
                 = MyfacesConfig.getCurrentInstance(facesContext.getExternalContext()).isStrictXhtmlLinks();
-        for (Iterator it = getChildren(linkComponent).iterator(); it.hasNext();) {
-            UIComponent child = (UIComponent) it.next();
-            if (child instanceof UIParameter) {
-                String name = ((UIParameter) child).getName();
-                Object value = ((UIParameter) child).getValue();
+        for (Iterator it = getChildren(linkComponent).iterator(); it.hasNext(); )
+        {
+            UIComponent child = (UIComponent)it.next();
+            if (child instanceof UIParameter)
+            {
+                String name = ((UIParameter)child).getName();
+                Object value = ((UIParameter)child).getValue();
                 addParameterToHref(name, value, hrefBuf, firstParameter, charEncoding, strictXhtmlLinks);
                 firstParameter = false;
             }
@@ -456,42 +477,69 @@ public abstract class HtmlLinkRendererBase
     }
 
     protected void renderOutputLinkStart(FacesContext facesContext, UIOutput output)
-        throws IOException {
+            throws IOException
+    {
         ResponseWriter writer = facesContext.getResponseWriter();
 
-        //calculate href
-        String href = org.apache.myfaces.shared_tomahawk.renderkit.RendererUtils.getStringValue(facesContext, output);
-        //check if there is an anchor # in it
-        int index = href.indexOf('#');
-        String anchorString = null;
-        if (index > -1)
+        if (HtmlRendererUtils.isDisabled(output))
         {
-            // remove anchor element and add it again after the parameter are encoded
-            anchorString = href.substring(index,href.length());
-            href = href.substring(0,index);
+            writer.startElement(HTML.SPAN_ELEM, output);
+            HtmlRendererUtils.writeIdIfNecessary(writer, output, facesContext);
+            HtmlRendererUtils.renderHTMLAttributes(writer, output, HTML.ANCHOR_PASSTHROUGH_ATTRIBUTES);
         }
-        if (getChildCount(output) > 0) {
-            StringBuffer hrefBuf = new StringBuffer(href);
-            addChildParametersToHref(facesContext, output, hrefBuf,
+        else
+        {
+            //calculate href
+            String href = org.apache.myfaces.shared_tomahawk.renderkit.RendererUtils.getStringValue(facesContext, output);
+            //check if there is an anchor # in it
+            int index = href.indexOf('#');
+            String anchorString = null;
+            if (index > -1)
+            {
+                // remove anchor element and add it again after the parameter are encoded
+                anchorString = href.substring(index,href.length());
+                href = href.substring(0,index);
+            }
+            if (getChildCount(output) > 0)
+            {
+                StringBuffer hrefBuf = new StringBuffer(href);
+                addChildParametersToHref(facesContext, output, hrefBuf,
                                      (href.indexOf('?') == -1), //first url parameter?
                                      writer.getCharacterEncoding());
-            href = hrefBuf.toString();
+                href = hrefBuf.toString();
+            }
+            if (index > -1)
+            {
+                href += anchorString;
+            }
+            href = facesContext.getExternalContext().encodeResourceURL(href);    //TODO: or encodeActionURL ?
+
+            //write anchor
+            writer.startElement(HTML.ANCHOR_ELEM, output);
+            HtmlRendererUtils.writeIdAndNameIfNecessary(writer, output, facesContext);
+            writer.writeURIAttribute(HTML.HREF_ATTR, href, null);
+            HtmlRendererUtils.renderHTMLAttributes(writer, output, org.apache.myfaces.shared_tomahawk.renderkit.html.HTML.ANCHOR_PASSTHROUGH_ATTRIBUTES);
+            writer.flush();
         }
-        if (index > -1)
+    }
+
+    private void renderLinkParameter(String name,
+                                     Object value,
+                                     StringBuffer onClick,
+                                     String jsForm,
+                                     UIComponent nestingForm)
+    {
+        if (name == null)
         {
-            href += anchorString;
+            throw new IllegalArgumentException("Unnamed parameter value not allowed within command link.");
         }
-        href = facesContext.getExternalContext().encodeResourceURL(href);    //TODO: or encodeActionURL ?
+        onClick.append(jsForm);
+        onClick.append(".elements['").append(name).append("']");
+        //UIParameter is no ValueHolder, so no conversion possible
+        String strParamValue = value != null ? org.apache.myfaces.shared_tomahawk.renderkit.html.util.HTMLEncoder.encode(value.toString(), false, false) : "";
+        onClick.append(".value='").append(strParamValue).append("';");
 
-        String clientId = output.getClientId(facesContext);
-
-        //write anchor
-        writer.startElement(HTML.ANCHOR_ELEM, output);
-        writer.writeAttribute(HTML.ID_ATTR, clientId, null);
-        writer.writeAttribute(HTML.NAME_ATTR, clientId, null);
-        writer.writeURIAttribute(HTML.HREF_ATTR, href, null);
-        HtmlRendererUtils.renderHTMLAttributes(writer, output, org.apache.myfaces.shared_tomahawk.renderkit.html.HTML.ANCHOR_PASSTHROUGH_ATTRIBUTES);
-        writer.flush();
+        addHiddenCommandParameter(FacesContext.getCurrentInstance(), nestingForm, name);
     }
 
     private static void addParameterToHref(String name,
@@ -516,7 +564,8 @@ public abstract class HtmlLinkRendererBase
 
         hrefBuf.append(URLEncoder.encode(name, charEncoding));
         hrefBuf.append('=');
-        if (value != null) {
+        if (value != null)
+        {
             //UIParameter is no ConvertibleValueHolder, so no conversion possible
             hrefBuf.append(URLEncoder.encode(value.toString(), charEncoding));
         }
@@ -524,16 +573,36 @@ public abstract class HtmlLinkRendererBase
 
 
     protected void renderOutputLinkEnd(FacesContext facesContext, UIComponent component)
-        throws IOException {
+            throws IOException
+    {
         ResponseWriter writer = facesContext.getResponseWriter();
-        // force separate end tag
-        writer.writeText("", null);
-        writer.endElement(HTML.ANCHOR_ELEM);
+
+        if (HtmlRendererUtils.isDisabled(component))
+        {
+            writer.endElement(HTML.SPAN_ELEM);
+        }
+        else
+        {
+            // force separate end tag
+            writer.writeText("", null);
+            writer.endElement(HTML.ANCHOR_ELEM);
+        }
     }
 
     protected void renderCommandLinkEnd(FacesContext facesContext, UIComponent component)
-        throws IOException {
-        renderOutputLinkEnd(facesContext, component);
+            throws IOException
+    {
+        ResponseWriter writer = facesContext.getResponseWriter();
+        if (HtmlRendererUtils.isDisabled(component))
+        {
+
+            writer.endElement(HTML.SPAN_ELEM);
+        }
+        else
+        {
+            writer.writeText("", null);
+            writer.endElement(HTML.ANCHOR_ELEM);
+        }
     }
 
 }
